"""Classes that inherit from PropertiedClass can use attribute names 
   to trigger actions rather than to simply get or 
   set the attribute. (Similar to a property in Visual Basic, or a 
   parameterless method in Eiffel).

   One simple use is for attribute validation. This facility can also
   be used to make an attribute read-only or undeleteable.

   A property attribute is an attribute 'a' for which methods have been 
   supplied to handle any or all of these situations:
        x.a        ('get' the attribute)
        x.a = v    ('set' the attribute)
        del x.a.   ('delete' the attribute)
   
   A "handler" for each of these is a method in the class with the following
   signatures:
       aget (self, name)  
       aset (self, name, value) 
       adel (self, name) 

   An attribute that is not a property is a 'basic' attribute and is handled
   by direct manipulation of the instance's __dict__.
 
   Routines _basic_get, _basic_set, and _basic_del are available to use
   when you wish to avoid the property handler. These
   three methods use the instance dictionary directly.

   Usage:
       Have your class inherit from PropertiedClass.

       Use methods _basic_set, _basic_get, _basic_del to evade your own 
       handlers when necessary.

       After class is defined, make calls to set_property. This is
       available as a function or method. 
   
       A child can override a parent's handler for a given name.

      If a class inherits from PropertiedClass and wishes to redefine 
      __getattr__, __setattr__, or __delattr__, great care is needed.
      The property behavior will be lost unless the PropertiedClass versions
      are invoked in the new version.
"""
import sys, types

def initialize_property_class (C):
    """
    Turn on the property handler in class C.
    """
    if not issubclass(C, PropertiedClass):
        raise TypeError, \
              'Argument must be a class inheriting from PropertiedClass'
    # Note that we have to make sure C itself has properties dicts, not one
    # of its parents.
    if C.__dict__.has_key('_properties_g'):
        return
# The _properties_x map names to handlers.
# They are filled in with those from the parent classes.
    C._properties_g = {}
    C._properties_s = {}
    C._properties_d = {}
    for P in C.__bases__:
        if issubclass(P, PropertiedClass):
            if P is PropertiedClass: continue  #no need for it, no handlers.
            initialize_property_class(P) 
            for name in P._properties_g.keys():
                if not C._properties_g.has_key(name):
                    C._properties_g[name] = P._properties_g[name]
            for name in P._properties_s.keys():
                if not C._properties_s.has_key(name):
                    C._properties_s[name] = P._properties_s[name]
            for name in P._properties_d.keys():
                if not C._properties_d.has_key(name):
                    C._properties_d[name] = P._properties_d[name]


def set_property (C, name, actg=None, acts=None, actd=None,
                           nowrite=None, nodelete=None):
    """Set attribute handlers for  name to methods actg, acts, actd
       None means no change for that action.
       nowrite = 1 prevents setting this attribute.
           nowrite defaults to 0. 
       nodelete = 1 prevents deleting this attribute.
           nodelete defaults to 1 unless actd given.
       if nowrite and nodelete is None: nodelete = 1
    """
    initialize_property_class (C)
    for h in (actg, acts, actd):
        if h is None: continue
        if not isinstance(h, types.UnboundMethodType):
            raise ValueError, 'Handlers must be unbound methods in the class'
    if nowrite and nodelete is None: 
        nodelete = 1
    if nowrite and acts is not None:
        raise ValueError, 'nowrite not consistent with acts specification.'
    if nodelete and actd is not None:
        raise ValueError, 'nodelete not consistent with actd specification.'
    if nowrite: 
        acts = C._write_disallowed
    if nodelete: 
        actd = C._delete_disallowed
    if actg is not None: 
        C._properties_g[name] = actg
    if acts is not None: 
        C._properties_s[name] = acts
    if actd is not None: 
        C._properties_d[name] = actd

def get_property_g (C, name):
    """Return get handler for name in C assuming it is not internal"""
    initialize_property_class (C)
    try:
        d = C._properties_g
        f = d[name]
        return f
    except AttributeError:  #no properties 
        return None
    except KeyError:
        return None
        
def get_property_s (C, name):
    """Return set handler for name in C assuming it is not internal"""
    initialize_property_class (C)
    try:
        d = C._properties_s
        return d[name]
    except AttributeError:  #no properties 
        return None
    except KeyError:
        return None
        
def get_property_d (C, name):
    """Return del handler for name in C assuming it is not internal"""
    initialize_property_class (C)
    try:
        d = C._properties_d
        return d[name]
    except AttributeError:  #no properties 
        return None
    except KeyError:
        return None
        
def get_properties (C, name=None):
    """Return dictionary mapping names to the handlers for class C; 
       if name given, just for that name.
    """
    if not issubclass(C, PropertiedClass): 
        return []
    initialize_property_class (C)
    names = []
    if name is None:
        for d in (C._properties_g, 
                  C._properties_s, 
                  C._properties_d
                 ):
            for n in d.keys():
                if n not in names: 
                    names.append(n)
    else:
        names.append(name)
    result = {}
    for n in names:
         result[n] = (get_property_g(C, n), 
                      get_property_s(C, n),
                      get_property_d(C, n)
                     )
    return result

def show_properties(C, name=None):
    """Print the result of get_properties(C, name) in a table."""
    if not isinstance(C, types.ClassType):
        try:
            C = C.__class__
        except AttributeError:
            print 'Object given is not an instance or class and hence has no property handlers.'
            return
    r = get_properties(C, name)
    names = r.keys()
    names.sort()
    for n in names:
        print n
        h = r[n]
        if h[0] is not None:
            print "    get", h[0]
        if h[1] is not None:
            print "    set", h[1]
        if h[2] is not None:
            print "    del", h[2]

class PropertiedClass:
    def __getattr__ (self, name):
        g = self.get_property_g (name)
        if g is None:
            return self._basic_get(name)
        else:
            return g(self, name)

    def __setattr__ (self, name, value):
        s = self.get_property_s (name)
        if s is None:
            self._basic_set(name, value)
        else:
            s(self, name, value)
   
    def __delattr__ (self, name): 
        d = self.get_property_d (name)
        if d is None:
            return self._basic_del(name)
        else:
            d(self, name)

    def _basic_get (self, name):
        """Fundamental attribute getter. 
           Do not invoke directly on private name.
           Evades property mechansim of this class.
        """
        try:
            return self.__dict__[name]
        except KeyError:
            raise AttributeError, "%s instance has no attribute %s" %\
                                      (self.__class__.__name__, name)

    def _basic_set (self, name, value):
        """Fundamental attribute setter. 
           Do not invoke directly on private name.
           Evades property mechanism of this class.
        """
        self.__dict__[name] = value

    def _basic_del (self, name):
        """Fundamental attribute deleter. 
           Do not invoke directly on private name.
           Evades property mechanism of this class.
        """
        try:
            del self.__dict__[name]
        except KeyError:
            raise AttributeError, "%s instance has no attribute %s" %\
                                      (self.__class__.__name__, name)

    def is_internal_attribute (self, name):
        """If self.is_internal_attribute(name), name does not use the 
           property handler. Default version: any name that starts with _."""
        return name[0] == '_'

    def _write_disallowed(self, name, value=None):
        "Use this as an acts to prevent write of an attribute."
        raise AttributeError, \
              'Write forbidden on %s in class %s.' % \
              (name, self.__class__.__name__,)

    def _delete_disallowed(self, name, value=None):
        "Use this as an actd to prevent delete of an attribute."
        raise AttributeError, \
              'Write forbidden on %s in class %s.' % \
              (name, self.__class__.__name__,)

    def set_property (self, name, actg=None, acts=None, actd=None,
                      nowrite=None, nodelete=None):
        """Set attribute handlers for  name to methods actg, acts, actd
           None means no change for that action.
           nowrite = 1 prevents setting this attribute.
               nowrite defaults to 0. 
           nodelete = 1 prevents deleting this attribute.
               nodelete defaults to 1 unless actd given.
           if nowrite and nodelete is None: nodelete = 1
        """
        set_property (self.__class__, name, actg, acts, actd, nowrite, 
                      nodelete)

    def get_property_g (self, name):
        """Return the 'get' property handler for name that self uses.
           Returns None if no handler.
        """
        if self.is_internal_attribute (name):
            return None
        else:
            return get_property_g(self.__class__, name)

    def get_property_s (self, name):
        """Return the 'set' property handler for name that self uses.
           Returns None if no handler.
        """
        if self.is_internal_attribute (name):
            return None
        else:
            return get_property_s(self.__class__, name)

    def get_property_d (self, name):
        """Return the 'del' property handler for name that self uses.
           Returns None if no handler.
        """
        if self.is_internal_attribute (name):
            return None
        else:
            return get_property_d(self.__class__, name)

PropertiedClass.__doc__ = __doc__



#! /usr/bin/env python
"""
Displaying image files in a separate thread on Tk+thread, w/ xv in
forked & execv'ed processes otherwise.

view(array):  will spawn a displaying program for arrays which are
              either NxM or NxMx3.  does the 'min/max' and conversion
              to char.

array2ppm(array): given an NxM or NxMx3 array, returns a ppm string
                  which is a valid thing to put in a PPM file.  (or
                  PGM file if NxM file).

TODO:
  - automatic scaling for small images
  - accept rank-1 arrays


"""
PPMVIEWER = 'xv' # configure
DEFAULT_HEIGHT = 255
MINSIZE = 150
import os
from Numeric import *
import tempfile, time
try:
    import Tkinter
    _have_tkinter = 1
    try:
        import Image
        _have_PIL = 1
        _have_tkImaging = 1# assume until proven wrong
    except ImportError:
        _have_PIL = 0
        _have_tkImaging = 0
except ImportError:
    _have_tkinter = 0
try:
    from threading import *
    _have_threads = 1
except ImportError:
    _have_threads = 0

def save_ppm(ppm, fname=None):
    if fname == None:
        fname = tempfile.mktemp('.ppm')
    f = open(fname, 'wb')
    f.write(ppm)
    f.close()
    return fname


def array2ppm(image):
    # scaling
    if len(image.shape) == 2:
        # B&W:
        image = transpose(image)
        return "P5\n#PPM version of array\n%d %d\n255\n%s" % \
               (image.shape[1], image.shape[0], ravel(image).tostring())
    else:
        # color
        image = transpose(image, (1, 0, 2))
        return "P6\n%d %d\n255\n%s" % \
               (image.shape[1], image.shape[0], ravel(image).tostring())

def preprocess(image, (scalex,scaley)):
    assert len(image.shape) in (1, 2) or \
           len(image.shape) == 3 and image.shape[2] == 3, \
           "image not correct format"
    themin = float(minimum.reduce(ravel(image)))
    themax = float(maximum.reduce(ravel(image)))
    if len(image.shape) == 1:
        len_x = image.shape[0]
        ys = ((image - themin)/(themax-themin)*(DEFAULT_HEIGHT-1)).astype('b')
        image = (zeros((DEFAULT_HEIGHT, len_x))+255).astype('b')
        for x in range(len_x):
            image[DEFAULT_HEIGHT-1-ys[x],len_x-x-1] = 0
        image = transpose(image)
    elif image.typecode() != 'b':
        image = (image - themin) / (themax-themin) * 255
        image = image.astype('b')

    len_x, len_y = image.shape[:2]
    if scalex is None:
        if len_x < MINSIZE:
            scalex = int(float(MINSIZE) / len_x) + 1
        else:
            scalex = 1
    if scaley is None:
        if len_y < MINSIZE:
            scaley = int(float(MINSIZE) / len_y) + 1
        else:
            scaley = 1
    return image, (scalex, scaley)

#----
# threaded stuff starts here
#----

import sys

_inidle = type(sys.stdin) == types.InstanceType and \
	  sys.stdin.__class__.__name__ == 'PyShell'

if _have_tkinter and (_have_threads or _inidle):

    def tk_root():
	if Tkinter._default_root is None:
	    root = Tkinter.Tk()
	    Tkinter._default_root.withdraw()
	else:
	    root = Tkinter._default_root
	return root

    _root = tk_root()


    class PILImage(Tkinter.Label):
        def __init__(self, master, data, (scalex, scaley)):
            width, height = data.shape[:2]
            if len(data.shape) == 3:
                mode = rawmode = 'RGB'
                bits = transpose(data, (1,0,2)).tostring()
            else:
                mode = rawmode = 'L'
                bits = transpose(data, (1,0)).tostring()
            self.image2 = Image.fromstring(mode, (width, height),
                                          bits, "raw", rawmode)
            import ImageTk
            self.image = ImageTk.PhotoImage(self.image2)
            Tkinter.Label.__init__(self, master, image=self.image,
                                   bg='black', bd=0)

    class PPMImage(Tkinter.Label):
        def __init__(self, master, ppm, (scalex, scaley)):
            self.image = Tkinter.PhotoImage(file=save_ppm(ppm))
            w, h = self.image.width(), self.image.height()
            self.image = self.image.zoom(scalex, scaley)
            self.image.configure(width=w*scalex, height=h*scaley)
            Tkinter.Label.__init__(self, master, image=self.image,
                                   bg="black", bd=0)
            
            self.pack()


    class ThreadedTk(Thread):

        def __init__(self, *args, **kw):
            self._done = 0

            apply(Thread.__init__, (self,)+args, kw)

        def done(self):
            self._done = 1

        def run(self):
            global _have_tkImaging
            while not self._done:
                _pendinglock.acquire()
                if len(_pendingarrays):       # there are files to process
                    for image, scales in _pendingarrays:
                        tl = Tkinter.Toplevel(background='black')
                        if _have_tkImaging:
                            try:
                                u = PILImage(tl, image, scales)
                            except Tkinter.TclError:
                                print "Error loading tkImaging"
                                _have_tkImaging = 0
                                u = PPMImage(tl, array2ppm(image), scales)
                        else:
                            u = PPMImage(tl, array2ppm(image), scales)
                        u.pack(fill='both', expand=1)
			u.tkraise()
                    del _pendingarrays[:]   # we're done
                _pendinglock.release()
                _root.update()  # do your thing
                time.sleep(0.01)   # go to sleep little baby

    def view(image, scale=(None,None)):
        global _have_tkImaging
        image, scales = preprocess(image, scale)
	if _inidle:
            tl = Tkinter.Toplevel()
      	    if _have_tkImaging:
                try:
                    u = PILImage(tl, image, scales)
                except Tkinter.TclError:
                    print "Error loading tkImaging"
                    _have_tkImaging = 0
                    u = PPMImage(tl, array2ppm(image), scales)
            else:
                u = PPMImage(tl, array2ppm(image), scales)
            u.pack(fill='both', expand=1)
            u.tkraise()
	else:
	    _pendinglock.acquire()
	    _pendingarrays.append((image, scales))
	    _pendinglock.release()
	    while len(_pendingarrays):
		time.sleep(0.01)

    if _inidle:
        def done(*args): pass
    else:
	_pendingarrays = []
	_pendinglock = Lock()
	_t = ThreadedTk() # this starts a Tk interpreter in a separate thread
	_t.start()
	done = _t.done
else:
    import sys
    # we're either w/o tk or w/o threads, so we hope we're on unix.
    if sys.platform == 'win32':
	if not _have_tkinter:
	    if not _threads:
		raise 'ConfigurationError', "view needs Tkinter on Win32, and either threads or the IDLE editor"
	    elif not _inidle:
		raise 'ConfigurationError', "view needs either threads or the IDLE editor to be enabled"
    children = []
    def view(image):
        global children
        image, scales = preprocess(image, (None,None))
	try:
	    pid = os.fork()
	    if pid == 0:
		ppm = array2ppm(image)
                try:
                    ret = os.system('%s %s' % (PPMVIEWER, save_ppm(ppm)))
                    if ret != 0:
                        raise 'ConfigurationError', "PPM image viewer '%s' not found" %PPMVIEWER
                except:
                    raise 'ConfigurationError', "PPM image viewer '%s' not found" %PPMVIEWER
	    else:
		children.append(pid)
	except:
            print "Your system has neither threads nor 'fork'."
            print "As a result, this program can't run interactively."
            print "We'll spawn the image viewer and exit."
            ppm = array2ppm(image)
            os.system('%s %s' % (PPMVIEWER, save_ppm(ppm)))
    def done():
        import signal
        for pid in children:
            os.kill(pid, signal.SIGQUIT)

# this little bit cleans up
import sys
if hasattr(sys, 'exitfunc'):
    oldexitfunc = sys.exitfunc
else:
    oldexitfunc = None
def cleanup():
    done()
    if oldexitfunc is not None:
	oldexitfunc()
sys.exitfunc = cleanup

if __name__ == '__main__':
    phase = 0.0
    while phase != -1:
        x = sin(arange(0+phase, 6+phase, .1))
        view(x)
        y = cos(2*arange(0+phase, 6+phase, .1))
        z = x[:,NewAxis] + y[NewAxis,:]
        z2 = x[::-1,NewAxis] + y[NewAxis,:]
        z3 = x[:,NewAxis] + y[NewAxis,::-1]
        view(z)
        z = transpose(array((z,z2,z3)), (2,1,0))
        view(z)
        phase = input("phase (-1 to quit) = ")



import string
import types

import Numeric
import LinearAlgebra
from UserArray import UserArray, asarray
from Numeric import dot, identity
from LinearAlgebra import inverse

# make translation table
_table = [None]*256
for k in range(256):
    _table[k] = chr(k)
_table = ''.join(_table)

_numchars = string.digits + ".-+jeEL"
_todelete = []
for k in _table:
    if k not in _numchars:
        _todelete.append(k)
_todelete = ''.join(_todelete)

def _eval(astr):
    return eval(astr.translate(_table,_todelete))

def _convert_from_string(data):
    data.find
    rows = data.split(';')
    newdata = []
    count = 0
    for row in rows:
        trow = row.split(',')
        newrow = []
        for col in trow:
            temp = col.split()
            newrow.extend(map(_eval,temp))
        if count == 0:
            Ncols = len(newrow)
        elif len(newrow) != Ncols:
            raise ValueError, "Rows not the same size."
        count += 1
        newdata.append(newrow)
    return newdata
        

class Matrix(UserArray):
    def __init__(self, data, typecode=None):
        if type(data) is types.StringType:
            data = _convert_from_string(data)
        UserArray.__init__(self,data,typecode)
        if len(self.array.shape) == 1:
            self.array.shape = (1,self.array.shape[0])
        self.shape = self.array.shape

    def __mul__(self, other):
        aother = asarray(other)
        if len(self.array) == 0:
            return self._rc(self.array*aother)
        else:
            return self._rc(dot(self.array, aother))

    def __rmul__(self, other):
        aother = asarray(other)
        if len(aother.shape) == 0:
            return self._rc(aother*self.array)
        else:
            return self._rc(dot(aother, self.array))

    def __pow__(self, other):
        shape = self.array.shape
        if len(shape)!=2 or shape[0]!=shape[1]:
            raise TypeError, "matrix is not square"
        if type(other) in (type(1), type(1L)):
            if other==0:
                return Matrix(identity(shape[0]))
            if other<0:
                result=Matrix(inverse(self.array))
                x=Matrix(result)
                other=-other
            else:
                result=self
                x=result
            while(other>1):
                result=result*x
                other=other-1
            return result
        else:
            raise TypeError, "exponent must be an integer"        

    def __rpow__(self, other):
	raise TypeError, "x**y not implemented for matrices y"

    def __invert__(self):
        return Matrix(Numeric.conjugate(self.array))
    
    def __setattr__(self,attr,value):
        if attr=='shape':
            if len(value) == 1:
                value = value + (1,)
            if len(value) != 2:
                raise ValueError, "Can only reshape a Matrix as a 2-d array."
            self.array.shape=value
        self.__dict__[attr]=value

    def __getattr__(self, attr):
        if attr == 'A':
            return self.array
        elif attr == 'T':
            return Matrix(Numeric.transpose(self.array))
        elif attr == 'H':
            if len(self.array.shape) == 1:
                self.array.shape = (1,self.array.shape[0])
            return Matrix(Numeric.conjugate(Numeric.transpose(self.array)))
        elif attr == 'real':
            return Matrix(self.array.real)
        elif attr == 'imag':
            return Matrix(self.array.imag)
        elif attr == 'flat':
            return Matrix(self.array.flat)
        else:
            raise AttributeError, attr + " not found."
        
    def asarray(self,t=None):
        if t is None:
            return self.array
        else:
            return asarray(self.array,t)


if __name__ == '__main__':
	from Numeric import *
	m = Matrix( [[1,2,3],[11,12,13],[21,22,23]])
	print m*m
	print m.array*m.array
	print transpose(m)
	print m**-1
        m = Matrix([[1,1],[1,0]])
        print "Fibonacci numbers:",
        for i in range(10):
            mm=m**i
            print mm[0][0],
        print
        
            
            



"""Reference implementation of PEP 0242"""
import PropertiedClasses
import _kinds

class KindObject (PropertiedClasses.PropertiedClass):
    """A KindObject is a constructor for certain varieties of integers,
     floating point numbers, and complex numbers.
    """
    def __init__ (self, **meta):
        for k, v in meta.items():
            self._basic_set(k, v)
            self.set_property(k, nowrite=1)

    def str (self):
        return '<KindObject named ' + self.name + '>'

class IntKindObject (KindObject):
    "Kind object for standard integers, corresponding to C ints."
    def __init__(self):
        "Create the kind object for standard integers."
        KindObject.__init__(self, 
                         name='int', 
                         typecode='i', 
                         MAX=_kinds.INT_MAX,
                         MIN=_kinds.INT_MIN
                         )
    def __call__ (self, n):
        "Convert to a standard integer."
        try:
            return int(n)
        except Exception, e:
            raise OverflowError, 'Could not convert to standard integer.'

class LongKindObject (KindObject):
    "Kind object for Python long -- no corresponding C type."
    def __init__(self):
        "Create the kind object for standard integers."
        KindObject.__init__(self, 
                         name='long', 
                         typecode='L', 
                         MAX=None,
                         MIN=None
                         )
    def __call__ (self, n):
        "Convert to a standard integer."
        try:
            return long(n)
        except Exception, e:
            raise OverflowError, 'Could not convert to standard integer.'

class DoubleKindObject (KindObject):
    "Kind object for a floating type corresponding to C double."
    def __init__ (self):
        """Create the kind object corresponding to a C double, standard Python real."""
        KindObject.__init__ (self,
                         name='double', 
                         typecode='d',
                         MAX=_kinds.DBL_MAX, 
                         MIN=_kinds.DBL_MIN,
                         DIG=_kinds.DBL_DIG,
                         EPSILON=_kinds.DBL_EPSILON,
                         MAX_EXP=_kinds.DBL_MAX_EXP,
                         MIN_EXP=_kinds.DBL_MIN_EXP,
                         MAX_10_EXP=_kinds.DBL_MAX_10_EXP,
                         MIN_10_EXP=_kinds.DBL_MIN_10_EXP,
                         RADIX=_kinds.FLT_RADIX,
                         ROUNDS=_kinds.FLT_ROUNDS)

    def __call__ (self, x):
        """Convert to a standard Python real."""
        try:
            return float(x)
        except Exception, e:
            raise OverflowError, e

class DoubleComplexKindObject (KindObject):
    "Kind object for a floating type corresponding to C doublecomplex."
    def __init__ (self):
        "Create a complex kind object using the given float kind and conversion routine."
        KindObject.__init__(self,
                            name='doublecomplex',
                            typecode='D'
                            )
        self.__floatkind = DoubleKindObject()

    def float_kind(self):
        return self.__floatkind

    def __call__ (self, x, y=None):
        "Convert to a standard Python complex."
        try:
            if y is None:
                return complex(x)
            else:
                return complex(x,y)
        except Exception, e:
            raise OverflowError, e

# this is just here as a starting point for future expansion...
class FloatKindObject (KindObject):
    "Kind object for a floating type corresponding to C float."
    def __init__ (self):
        """Create the kind object corresponding with C float."""
        raise RuntimeError, 'float kind not supported'
        KindObject.__init__ (self,
                         name='float', 
                         typecode='f',
                         MAX=_kinds.FLT_MAX, 
                         MIN=_kinds.FLT_MIN,
                         DIG=_kinds.FLT_DIG,
                         EPSILON=_kinds.FLT_EPSILON,
                         MAX_EXP=_kinds.FLT_MAX_EXP,
                         MIN_EXP=_kinds.FLT_MIN_EXP,
                         MAX_10_EXP=_kinds.FLT_MAX_10_EXP,
                         MIN_10_EXP=_kinds.FLT_MIN_10_EXP,
                         RADIX=_kinds.FLT_RADIX,
                         ROUNDS=_kinds.FLT_ROUNDS)

    def __call__ (self, x):
        """Convert to floating type corresponding to C float."""
        try:
            return float32(x) # does not exist in standard release
        except Exception, e:
            raise OverflowError, e

__intkind = IntKindObject()
__longkind = LongKindObject()
__doublekind = DoubleKindObject()
__doublecomplexkind = DoubleComplexKindObject()

int_kinds = [__intkind, __longkind]
float_kinds = [__doublekind]
complex_kinds = [__doublecomplexkind]

default_int_kind=__intkind
default_long_kind = __longkind
default_float_kind = __doublekind
default_complex_kind = __doublecomplexkind

def int_kind (n):
    """Return a kind object corresponding to an integer kind holding numbers
       with n decimal digits.
       If n is zero, return default_int_kind.
    """
    if n < 0: 
        raise ValueError, 'int_kind argument must be non-negative'
    elif n == 0: 
        return default_int_kind
    nl = 10L**(n)-1
    for ikind in int_kinds[:-1]:
        if nl <= long(ikind.MAX) and \
           -nl >= long(ikind.MIN):
            return ikind
    else:
        return default_long_kind


def float_kind(nd, n):
    """Return a floating kind object with at least nd digits of precision 
       that will hold numbers in the range [-10**n, 10**n], in which
       1.e-n is not zero.

       If nd = n = 0, return kind corresponding to Python default floating type.
    """
    if nd < 0 or n <0:
        raise ValueError, 'float_kind arguments must be non-negative.'
    if n == 0 and nd == 0:
        return default_float_kind
    for akind in float_kinds:
        if nd <= akind.DIG and \
            n <=  akind.MAX_10_EXP and \
            -n >= akind.MIN_10_EXP:
            return akind
    else:
        raise OverflowError, 'No such floating kind available.'

def complex_kind(nd, n):
    """Return a complex kind object with at least nd digits of precision 
       that will hold numbers in the range [-10**n, 10**n], in which
       1.e-n is not zero.
    """
    if nd < 0 or n <0:
        raise ValueError, 'complex_kind arguments must be non-negative.'
    if n == 0 and nd == 0:
        return default_complex_kind
    for ckind in complex_kinds:
        akind = ckind.floatkind
        if nd <= akind.DIG and \
            n <=  akind.MAX_10_EXP and \
            -n >= akind.MIN_10_EXP:
            return ckind  
    else:
        raise OverflowError, 'No such complex kind available.'




"""Numeric module defining a multi-dimensional array and useful procedures for
   Numerical computation.

Functions

-   array                      - NumPy Array construction
-   zeros                      - Return an array of all zeros
-   fromstring                 - Construct array from (byte) string
-   take                       - Select sub-arrays using sequence of indices
-   put                        - Set sub-arrays using sequence of 1-D indices
-   putmask                    - Set portion of arrays using a mask 
-   reshape                    - Return array with new shape
-   repeat                     - Repeat elements of array
-   choose                     - Construct new array from indexed array tuple
-   cross_correlate            - Correlate two 1-d arrays
-   searchsorted               - Search for element in 1-d array
-   sum                        - Total sum over a specified dimension
-   cumsum                     - Cumulative sum over a specified dimension
-   product                    - Total product over a specified dimension
-   cumproduct                 - Cumulative product over a specified dimension
-   alltrue                    - Logical and over an entire axis
-   sometrue                   - Logical or over an entire axis
-   allclose		       - Tests if sequences are essentially equal

More Functions:

-   arrayrange (arange)        - Return regularly spaced array
-   asarray                    - Guarantee NumPy array
-   sarray                     - Guarantee a NumPy array that keeps precision 
-   convolve                   - Convolve two 1-d arrays
-   swapaxes                   - Exchange axes
-   concatenate                - Join arrays together
-   transpose                  - Permute axes
-   sort                       - Sort elements of array
-   argsort                    - Indices of sorted array
-   argmax                     - Index of largest value                      
-   argmin                     - Index of smallest value
-   innerproduct               - Innerproduct of two arrays
-   dot                        - Dot product (matrix multiplication)
-   outerproduct               - Outerproduct of two arrays
-   resize                     - Return array with arbitrary new shape
-   indices                    - Tuple of indices
-   fromfunction               - Construct array from universal function
-   diagonal                   - Return diagonal array
-   trace                      - Trace of array
-   dump                       - Dump array to file object (pickle)
-   dumps                      - Return pickled string representing data
-   load                       - Return array stored in file object
-   loads                      - Return array from pickled string
-   ravel                      - Return array as 1-D 
-   nonzero                    - Indices of nonzero elements for 1-D array
-   shape                      - Shape of array
-   where                      - Construct array from binary result
-   compress                   - Elements of array where condition is true
-   clip                       - Clip array between two values
-   zeros                      - Array of all zeros
-   ones                       - Array of all ones
-   identity                   - 2-D identity array (matrix)

(Universal) Math Functions 

       add                    logical_or             exp        
       subtract               logical_xor            log        
       multiply               logical_not            log10      
       divide                 maximum                sin        
       divide_safe            minimum                sinh       
       conjugate              bitwise_and            sqrt       
       power                  bitwise_or             tan        
       absolute               bitwise_xor            tanh       
       negative               invert                 ceil       
       greater                left_shift             fabs       
       greater_equal          right_shift            floor      
       less                   arccos                 arctan2    
       less_equal             arcsin                 fmod       
       equal                  arctan                 hypot      
       not_equal              cos                    around     
       logical_and            cosh                   sign
       arccosh                arcsinh                arctanh

"""   

import numeric_version
__version__ = numeric_version.version
del numeric_version

import multiarray
from umath import * 
from Precision import *

import _numpy # for freeze dependency resolution (at least on Mac)

import string, types, math

#Use this to add a new axis to an array
NewAxis = None

#The following functions are considered builtin, they all might be
#in C some day

def arrayrange(start, stop=None, step=1, typecode=None):
    """Just like range() except it returns a array whose type can be specfied
    by the keyword argument typecode.
    """
    
    if (stop is None):
        stop = start
        start = 0
    n = int(math.ceil(float(stop-start)/step))
    if n <= 0:
        m = zeros( (0,) )+(step+start+stop)
    else:
        m = (add.accumulate(ones((n,), Int))-1)*step +(start+(stop-stop))
        # the last bit is to deal with e.g. Longs -- 3L-3L==0L
    if typecode is not None and m.typecode() != typecode:
        return m.astype(typecode)
    else:
        return m

array = multiarray.array
zeros = multiarray.zeros

def asarray(a, typecode=None, savespace=0):
    """asarray(a,typecode=None) returns a as a NumPy array.  Unlike array(),
    no copy is performed if a is already an array.
    """
    return multiarray.array(a, typecode, copy=0, savespace=savespace)

def sarray(a, typecode=None, copy=0):
    """sarray(a, typecode=None, copy=0) calls array with savespace=1."""
    return multiarray.array(a, typecode, copy, savespace=1)

fromstring = multiarray.fromstring
take = multiarray.take
reshape = multiarray.reshape
choose = multiarray.choose
cross_correlate = multiarray.cross_correlate

def repeat(a, repeats, axis=0):
    """repeat elements of a repeats times along axis
       repeats is a sequence of length a.shape[axis]
       telling how many times to repeat each element.
       If repeats is an integer, it is interpreted as
       a tuple of length a.shape[axis] containing repeats.
       The argument a can be anything array(a) will accept.
    """
    a = array(a, copy=0)
    s = a.shape
    if isinstance(repeats, types.IntType):
        repeats = tuple([repeats]*(s[axis]))
    if len(repeats) != s[axis]:
        raise ValueError, "repeat requires second argument integer or of length of a.shape[axis]."
    d = multiarray.repeat(a, repeats, axis)
    return d

def put (a, ind, v):
    """put(a, ind, v) results in a[n] = v[n] for all n in ind
       If v is shorter than mask it will be repeated as necessary.
       In particular v can be a scalar or length 1 array.
    """
    multiarray.put (a, ind, array(v, copy=0).astype(a.typecode()))

def putmask (a, mask, v):
    """putmask(a, mask, v) results in a = v for all places mask is true.
       If v is shorter than mask it will be repeated as necessary.
       In particular v can be a scalar or length 1 array.
    """
    tc = a.typecode()
    mask = asarray(mask).astype(Int)
    v = array(v, copy=0).astype(tc)
    if tc == PyObject:
        if v.shape == (): v.shape=(1,)
        ax = ravel(a)
        mx = ravel(mask)
        vx = ravel(v)
        vx = resize(vx, ax.shape)
        for i in range(len(ax)):
            if mx[i]: ax[i] = vx[i]
    else:
        multiarray.putmask (a, mask, v)

def convolve(a,v,mode=2):
    """Returns the discrete, linear convolution of 1-D
    sequences a and v; mode can be 0 (valid), 1 (same), or 2 (full)
    to specify size of the resulting sequence.
    """
    if (len(v) > len(a)):
        temp = a
        a = v
        v = temp
        del temp
    return cross_correlate(a,asarray(v)[::-1],mode)

ArrayType = multiarray.arraytype

def swapaxes(a, axis1, axis2):
    """swapaxes(a, axis1, axis2) returns array a with axis1 and axis2
    interchanged.
    """
    a = array(a, copy=0)
    n = len(a.shape)
    if n <= 1: return a
    if axis1 < 0: axis1 += n
    if axis2 < 0: axis2 += n
    if axis1 < 0 or axis1 >= n:
        raise ValueError, "Bad axis1 argument to swapaxes."
    if axis2 < 0 or axis2 >= n:
        raise ValueError, "Bad axis2 argument to swapaxes."
    new_axes = arange(n)
    new_axes[axis1] = axis2
    new_axes[axis2] = axis1
    return multiarray.transpose(a, new_axes)

arraytype = multiarray.arraytype
#add extra intelligence to the basic C functions
def concatenate(a, axis=0):
    """concatenate(a, axis=0) joins the tuple of sequences in a into a single
    NumPy array.
    """
    if axis == 0:
        return multiarray.concatenate(a)
    else:
        new_list = []
        for m in a:
            new_list.append(swapaxes(m, axis, 0))
    return swapaxes(multiarray.concatenate(new_list), axis, 0)

def transpose(a, axes=None):
    """transpose(a, axes=None) returns array with dimensions permuted
    according to axes.  If axes is None (default) returns array with
    dimensions reversed.
    """
    if axes is None:
        axes = arange(len(array(a).shape))[::-1]
    return multiarray.transpose(a, axes)

def sort(a, axis=-1):
    """sort(a,axis=-1) returns array with elements sorted along given axis.
    """
    a = array(a, copy=0)
    n = len(a.shape)
    if axis < 0: axis += n
    if axis < 0 or axis >= n:
        raise ValueError, "sort axis argument out of bounds"
    if axis != n-1: a = swapaxes(a, axis, n-1)
    s = multiarray.sort(a)
    if axis != n-1: s = swapaxes(s, axis, -1)
    return s

def argsort(a, axis=-1):
    """argsort(a,axis=-1) return the indices into a of the sorted array
    along the given axis, so that take(a,result,axis) is the sorted array.
    """
    a = array(a, copy=0)
    n = len(a.shape)
    if axis < 0: axis += n
    if axis < 0 or axis >= n:
        raise ValueError, "argsort axis argument out of bounds"
    if axis != n-1: a = swapaxes(a, axis, n-1)
    s = multiarray.argsort(a)
    if axis != n-1: s = swapaxes(s, axis, -1)
    return s

def argmax(a, axis=-1):
    """argmax(a,axis=-1) returns the indices to the maximum value of the
    1-D arrays along the given axis.    
    """
    a = array(a, copy=0)
    n = len(a.shape)
    if axis < 0: axis += n
    if axis < 0 or axis >= n:
        raise ValueError, "argmax axis argument out of bounds"
    if axis != n-1: a = swapaxes(a, axis, n-1)
    s = multiarray.argmax(a)
    if axis != n-1: s = swapaxes(s, axis, -1)
    return s

def argmin(a, axis=-1):
    """argmin(a,axis=-1) returns the indices to the minimum value of the
    1-D arrays along the given axis.    
    """
    a = -array(a, copy=0)
    n = len(a.shape)
    if axis < 0: axis += n
    if axis < 0 or axis >= n:
        raise ValueError, "argmin axis argument out of bounds" 
    if axis != n-1: a = swapaxes(a, axis, n-1)
    s = multiarray.argmax(a)
    if axis != n-1: s = swapaxes(s, axis, -1)
    return s


searchsorted = multiarray.binarysearch

def innerproduct(a,b):
    """innerproduct(a,b) returns the dot product of two arrays, which has
    shape a.shape[:-1] + b.shape[:-1] with elements computed by summing the
    product of the elements from the last dimensions of a and b.
    """
    try:
        return multiarray.innerproduct(a,b)
    except TypeError,detail:
        if array(a).shape == () or array(b).shape == ():
            return a*b
        else:
            raise TypeError, detail or "invalid types for dot"

def outerproduct(a,b):
   """outerproduct(a,b) returns the outer product of two vectors.
      result(i,j) = a(i)*b(j) when a and b are vectors
      Will accept any arguments that can be made into vectors.
   """
   return asarray(a).flat[:,NewAxis]*asarray(b).flat[NewAxis,:]

def dot(a, b):
    """dot(a,b) returns matrix-multiplication between a and b.  The product-sum
    is over the last dimension of a and the second-to-last dimension of b.
    """
    return innerproduct(a, swapaxes(b, -1, -2))

#This is obsolete, don't use in new code
matrixmultiply = dot

#Use Konrad's printing function (modified for both str and repr now)
from ArrayPrinter import array2string
def array_repr(a, max_line_width = None, precision = None, suppress_small = None):
    return array2string(a, max_line_width, precision, suppress_small, ', ', 1)

def array_str(a, max_line_width = None, precision = None, suppress_small = None):
    return array2string(a, max_line_width, precision, suppress_small, ' ', 0)
    
multiarray.set_string_function(array_str, 0)
multiarray.set_string_function(array_repr, 1)

#This is a nice value to have around
#Maybe in sys some day
LittleEndian = fromstring("\001"+"\000"*7, 'i')[0] == 1

def resize(a, new_shape):
    """resize(a,new_shape) returns a new array with the specified shape.
    The original array's total size can be any size.
    """

    a = ravel(a)
    if not len(a): return zeros(new_shape, a.typecode())
    total_size = multiply.reduce(new_shape)
    n_copies = total_size / len(a)
    extra = total_size % len(a)

    if extra != 0: 
        n_copies = n_copies+1
        extra = len(a)-extra

    a = concatenate( (a,)*n_copies)
    if extra > 0:
        a = a[:-extra]

    return reshape(a, new_shape)

def indices(dimensions, typecode=None):
    """indices(dimensions,typecode=None) returns an array representing a grid
    of indices with row-only, and column-only variation.
    """
    tmp = ones(dimensions, typecode)
    lst = []
    for i in range(len(dimensions)):
        lst.append( add.accumulate(tmp, i, )-1 )
    return array(lst)

def fromfunction(function, dimensions):
    """fromfunction(function, dimensions) returns an array constructed by
    calling function on a tuple of number grids.  The function should
    accept as many arguments as there are dimensions which is a list of
    numbers indicating the length of the desired output for each axis.
    """
    return apply(function, tuple(indices(dimensions)))
    

def diagonal(a, offset= 0, axis1=0, axis2=1):
    """diagonal(a, offset=0, axis1=0, axis2=1) returns the given diagonals
    defined by the last two dimensions of the array.
    """
    a = array (a)
    if axis2 < axis1: axis1, axis2 = axis2, axis1
    if axis2 > 1:
        new_axes = range (len (a.shape))
        del new_axes [axis2]; del new_axes [axis1]
        new_axes [0:0] = [axis1, axis2]
        a = transpose (a, new_axes)
    s = a.shape
    if len (s) == 2:
        n1 = s [0]
        n2 = s [1]
        n = n1 * n2
        s = (n,)
        a = reshape (a, s)
        if offset < 0:
            return take (a, range ( - n2 * offset, min(n2, n1+offset) * (n2+1) - n2 * offset, n2+1), 0)
        else:
            return take (a, range (offset,         min(n1, n2-offset) * (n2+1) + offset,      n2+1), 0)
    else :
        my_diagonal = []
        for i in range (s [0]) :
            my_diagonal.append (diagonal (a [i], offset))
        return array (my_diagonal)

def trace(a, offset=0, axis1=0, axis2=1):
    """trace(a,offset=0, axis1=0, axis2=1) returns the sum along diagonals
    (defined by the last two dimenions) of the array.
    """
    return add.reduce(diagonal(a, offset, axis1, axis2))


# These two functions are used in my modified pickle.py so that
# matrices can be pickled.  Notice that matrices are written in 
# binary format for efficiency, but that they pay attention to
# byte-order issues for  portability.

def DumpArray(m, fp):    
    if m.typecode() == 'O': 
        raise TypeError, "Numeric Pickler can't pickle arrays of Objects"
    s = m.shape
    if LittleEndian: endian = "L"
    else: endian = "B"
    fp.write("A%s%s%d " % (m.typecode(), endian, m.itemsize()))
    for d in s:
        fp.write("%d "% d)
    fp.write('\n')
    fp.write(m.tostring())

def LoadArray(fp):
    ln = string.split(fp.readline())
    if ln[0][0] == 'A': ln[0] = ln[0][1:] # Nasty hack showing my ignorance of pickle
    typecode = ln[0][0]
    endian = ln[0][1]
    
    shape = map(lambda x: string.atoi(x), ln[1:])
    itemsize = string.atoi(ln[0][2:])

    sz = reduce(multiply, shape)*itemsize
    data = fp.read(sz)
        
    m = fromstring(data, typecode)
    m = reshape(m, shape)

    if (LittleEndian and endian == 'B') or (not LittleEndian and endian == 'L'):
        return m.byteswapped()
    else:
        return m

import pickle, copy
class Unpickler(pickle.Unpickler):
    def load_array(self):
        self.stack.append(LoadArray(self))
    
    dispatch = copy.copy(pickle.Unpickler.dispatch)    
    dispatch['A'] = load_array

class Pickler(pickle.Pickler):
    def save_array(self, object):
        DumpArray(object, self)

    dispatch = copy.copy(pickle.Pickler.dispatch)        
    dispatch[ArrayType] = save_array

#Convenience functions
from StringIO import StringIO

def dump(object, file):
    """dump(object, file) pickles (binary-writes) the object to an open file.
    """
    Pickler(file).dump(object)

def dumps(object):
    """dumps(object) pickles (binary-writes) the object and returns the byte
    stream.
    """
    file = StringIO()
    Pickler(file).dump(object)
    return file.getvalue()

def load(file):
    """load(file) returns an array from the open file pointing to pickled data. 
    """
    return Unpickler(file).load()

def loads(str):
    """loads(str) returns an array from a byte stream containing its pickled
    representation.
    """
    file = StringIO(str)
    return Unpickler(file).load()

# slightly different format uses the copy_reg mechanism
import copy_reg

def array_constructor(shape, typecode, thestr, Endian=LittleEndian):
    x = fromstring(thestr, typecode)
    x.shape = shape
    if LittleEndian != Endian:
        return x.byteswapped()
    else:
        return x

def pickle_array(a):
    return (array_constructor, 
            (a.shape, a.typecode(), a.tostring(), LittleEndian))

copy_reg.pickle(ArrayType, pickle_array, array_constructor)


# These are all essentially abbreviations
# These might wind up in a special abbreviations module

def ravel(m):
    """ravel(m) returns a 1d array corresponding to all the elements of it's
    argument.
    """
    return reshape(m, (-1,))

def nonzero(a):
    """nonzero(a) returns the indices of the elements of a which are not zero,
    a must be 1d
    """
    return repeat(arange(len(a)), not_equal(a, 0))

def shape(a):
    """shape(a) returns the shape of a (as a function call which
       also works on nested sequences).
    """
    return asarray(a).shape

def where(condition, x, y):
    """where(condition,x,y) is shaped like condition and has elements of x and
    y where condition is respectively true or false.
    """
    return choose(not_equal(condition, 0), (y, x))

def compress(condition, m, dimension=-1):
    """compress(condition, x, dimension=-1) = those elements of x corresponding 
    to those elements of condition that are "true".  condition must be the
    same size as the given dimension of x."""
    return take(m, nonzero(condition), dimension)

def clip(m, m_min, m_max):
    """clip(m, m_min, m_max) = every entry in m that is less than m_min is
    replaced by m_min, and every entry greater than m_max is replaced by
    m_max.
    """
    selector = less(m, m_min)+2*greater(m, m_max)
    return choose(selector, (m, m_min, m_max))

def ones(shape, typecode='l', savespace=0):
    """ones(shape, typecode=Int, savespace=0) returns an array of the given
    dimensions which is initialized to all ones. 
    """
    a=zeros(shape, typecode, savespace)
    a[...]=1
    return a
 
def identity(n,typecode='l'):
    """identity(n) returns the identity matrix of shape n x n.
    """
    return resize(array([1]+n*[0],typecode=typecode), (n,n))

def sum (x, axis=0):
    """Sum the array over the given axis.
    """
    x = array(x, copy=0)
    n = len(x.shape)
    if axis < 0: axis += n
    if n == 0 and axis in [0,-1]: return x[0]
    if axis < 0 or axis >= n:
        return ValueError, 'Improper axis argument to sum.'
    return add.reduce(x, axis)

def product (x, axis=0):
    """Product of the array elements over the given axis."""
    x = array(x, copy=0)
    n = len(x.shape)
    if axis < 0: axis += n
    if n == 0 and axis in [0,-1]: return x[0]
    if axis < 0 or axis >= n:
        return ValueError, 'Improper axis argument to product.'
    return multiply.reduce(x, axis)

def sometrue (x, axis=0):
    """Perform a logical_or over the given axis."""
    x = array(x, copy=0)
    n = len(x.shape)
    if axis < 0: axis += n
    if n == 0 and axis in [0,-1]: return x[0] != 0
    if axis < 0 or axis >= n:
        return ValueError, 'Improper axis argument to sometrue.'
    return logical_or.reduce(x, axis)

def alltrue (x, axis=0):
    """Perform a logical_and over the given axis."""
    x = array(x, copy=0)
    n = len(x.shape)
    if axis < 0: axis += n
    if n == 0 and axis in [0,-1]: return x[0] != 0
    if axis < 0 or axis >= n:
        return ValueError, 'Improper axis argument to product.'
    return logical_and.reduce(x, axis)

def cumsum (x, axis=0):
    """Sum the array over the given axis."""
    x = array(x, copy=0)
    n = len(x.shape)
    if axis < 0: axis += n
    if n == 0 and axis in [0,-1]: return x[0]
    if axis < 0 or axis >= n:
        return ValueError, 'Improper axis argument to cumsum.'
    return add.accumulate(x, axis)

def cumproduct (x, axis=0):
    """Sum the array over the given axis."""
    x = array(x, copy=0)
    n = len(x.shape)
    if axis < 0: axis += n
    if n == 0 and axis in [0,-1]: return x[0]
    if axis < 0 or axis >= n:
        return ValueError, 'Improper axis argument to cumproduct.'
    return multiply.accumulate(x, axis)

arange = arrayrange

def around(m, decimals=0):
    """around(m, decimals=0) \
    Round in the same way as standard python performs rounding. Returns 
    always a float.
    """
    m = asarray(m)
    s = sign(m)
    if decimals:
        m = absolute(m*10.**decimals)
    else:
        m = absolute(m)
    rem = m-asarray(m).astype(Int)
    m = where(less(rem,0.5), floor(m), ceil(m))
    # convert back
    if decimals:
        m = m*s/(10.**decimals)
    else:
        m = m*s
    return m
    
def sign(m):
    """sign(m) gives an array with shape of m with elements defined by sign
    function:  where m is less than 0 return -1, where m greater than 0, a=1,
    elsewhere a=0.
    """
    m = asarray(m)
    return zeros(shape(m))-less(m,0)+greater(m,0)

def allclose (a, b, rtol=1.e-5, atol=1.e-8):
    """ allclose(a,b,rtol=1.e-5,atol=1.e-8)
        Returns true if all components of a and b are equal
        subject to given tolerances.
        The relative error rtol must be positive and << 1.0
        The absolute error atol comes into play for those elements
        of y that are very small or zero; it says how small x must be also.
    """
    x = array(a, copy=0)
    y = array(b, copy=0)
    d = less(absolute(x-y), atol + rtol * absolute(y))
    return alltrue(ravel(d))
    
